#include "opnetProtoSimProcess.h"

 
extern "C"
{
#include    "oms_pr.h"
#include    "udp_api.h"
#include    "ip_addr_v4.h"
#include    "ip_rte_v4.h"
}

OpnetProtoSimProcess::OpnetProtoSimProcess() 
{
      
}

OpnetProtoSimProcess::~OpnetProtoSimProcess(void)
{
  
}

ProtoSimAgent::SocketProxy* OpnetProtoSimProcess::OpenSocket(ProtoSocket& theSocket)
{
#ifdef OP_DEBUG1
printf("\topnetProtosimProcess.pr.c - OpnetProtoSimProcess::OpenSocket() with udp_id = %d\n",
	udp_process_id);
#endif
    UdpSocketProxy* udpProxy = new UdpSocketProxy(udp_process_id);
    if (udpProxy)
    {
        udpProxy->AttachSocket(theSocket);
        socket_proxy_list.Prepend(*udpProxy);
        return udpProxy;             
    }    
    else
    {
        fprintf(stderr, "OpnetProtoSimProcess::OpenSocket() new SocketAgent error\n");
        return NULL;   
    }
}  // end OpnetProtoSimProcess::OpenSocket()

void OpnetProtoSimProcess::CloseSocket(ProtoSocket& theSocket)
{
#ifdef OP_DEBUG1
printf("\topnetProtosimProcess.pr.c - OpnetProtoSimProcess::CloseSocket()\n");
#endif
    SocketProxy* socketProxy = static_cast<SocketProxy*>(theSocket.GetHandle());
    // (TBD) Do we to do any OPNET cleanup here???
    socket_proxy_list.Remove(*socketProxy);
}  // end OpnetProtoSimProcess::CloseSocket()

void OpnetProtoSimProcess::OnReceive()
{
#ifdef OP_DEBUG1
printf("\topnetProtosimProcess.pr.c - OpnetProtoSimProcess::OnReceive()\n");
#endif
    // Get packet from stream ZERO
    Ici* cmd = op_intrpt_ici();
    if (OPC_NIL != cmd)
    {
        // 1) Get packet dest port and find which local socket it goes to
        int localPort;
        op_ici_attr_get(cmd, "local_port", &localPort);
        SocketProxy* socketProxy = socket_proxy_list.FindProxyByPort(localPort);
        if (socketProxy)
        {
            Packet* pkt = op_pk_get(0);
			// LP - 3-8-04 - added
//#ifdef OPNET_OLSR
//			record_receiving_stat(Pkt);
//#endif  // end LP
            // Get recv packet payload
            char* recvData;
            op_pk_fd_get(pkt, 0, &recvData);
            // Get recv packet length
            int recvLen  = op_pk_bulk_size_get(pkt) / 8;
            // Get source addr/port
            IpT_Address remoteAddr;
            op_ici_attr_get(cmd, "rem_addr", &remoteAddr);
            int remotePort;
            op_ici_attr_get(cmd, "rem_port", &remotePort);
            IpT_Address localAddr;
			
			op_ici_attr_get(cmd, "src_addr", &localAddr);
#ifdef OP_DEBUG1
			printf("\topnetProtosimProcess.pr.c - OpnetProtoSimProcess::OnReceive() - remoteAdd - %u, localAdrr = %u\n",
				remoteAddr, localAddr);
#endif			
            ProtoAddress srcAddr;
            srcAddr.SimSetAddress(remoteAddr);
            srcAddr.SetPort(remotePort);
            ProtoAddress dstAddr;
            dstAddr.SimSetAddress(localAddr);
            dstAddr.SetPort(localPort);
            // Pass packet content/info to socket proxy
            static_cast<UdpSocketProxy*>(socketProxy)->OnReceive(recvData, recvLen, srcAddr, dstAddr);
			op_prg_mem_free(recvData);
            op_pk_destroy(pkt); 
        }
    }
    else
    {
        // Generate an error and end simulation.  
	    op_sim_end ("Error: DoReceive() OPC_NIL cmd", "", "", "");
    }
}  // end OpnetProtoSimProcess::OnReceive()


OpnetProtoSimProcess::UdpSocketProxy::UdpSocketProxy(Objid udpProcessId)
 : udp_process_id(udpProcessId),
   mcast_ttl(255), mcast_loopback(false), 
   recv_data(NULL), recv_data_len(0)
{ 
#ifdef OP_DEBUG2
printf("\topnetProtosimProcess.pr.c - OpnetProtoSimProcess::UdpSocketProxy::UdpSocketProxy(udpId = %ld)\n",
	udpProcessId);
#endif
}

OpnetProtoSimProcess::UdpSocketProxy::~UdpSocketProxy()
{
    // (TBD) cleanup ???
}

bool OpnetProtoSimProcess::UdpSocketProxy::Bind(UINT16& thePort)
{
#ifdef OP_DEBUG1
printf("\topnetProtosimProcess.pr.c - OpnetProtoSimProcess::UdpSocketProxy::Bind, udp_process_id = %d)\n",
	udp_process_id);
#endif
    // (TBD) What happens if (0 == thePort) ???
    Ici* cmd = op_ici_create("udp_command_v3");
    op_ici_attr_set(cmd, "local_port", (int)thePort);
    op_ici_install(cmd);
    op_intrpt_force_remote(UDPC_COMMAND_CREATE_PORT, udp_process_id);   
    return true;       
}  // end OpnetProtoSimProcess::SocketAgent::Bind()

bool OpnetProtoSimProcess::UdpSocketProxy::SendTo(const char*         buffer, 
                                                  unsigned int&       buflen,
                                                  const ProtoAddress& dstAddr)
{
#ifdef OP_DEBUG1
printf("\tOpnetProtoSimProcess::UdpSocketProxy::SendTo: local_port %d, rem_port %d, rem_addr_%u\n",
		proto_socket->GetPort(), dstAddr.GetPort(), dstAddr.SimGetAddress()); // LP 2-10-04
#endif
    // (TBD) error check Opnet API calls ...
    Packet* pkt = op_pk_create(buflen*8);
    char* payload = (char*) op_prg_mem_copy_create((void*)buffer, buflen);
    op_pk_fd_set(pkt, 0, OPC_FIELD_TYPE_STRUCT, payload, 0,
                 op_prg_mem_copy_create, op_prg_mem_free, buflen);
    Ici* cmd = op_ici_create("udp_command_v3");
    op_ici_attr_set(cmd, "local_port", proto_socket->GetPort());
    op_ici_attr_set(cmd, "rem_port", dstAddr.GetPort());
    op_ici_attr_set(cmd, "rem_addr", dstAddr.SimGetAddress());
    op_ici_install(cmd);
    op_pk_send_forced(pkt, 0);
    return true;
}  // end OpnetProtoSimProcess::UdpSocketProxy::SendTo()

bool OpnetProtoSimProcess::UdpSocketProxy::RecvFrom(char*         buffer, 
                                                    unsigned int& numBytes, 
                                                    ProtoAddress& srcAddr)
{
#ifdef OP_DEBUG1
printf("\tOpnetProtoSimProcess::UdpSocketProxy::RecvFrom\n"); 
#endif
    if (recv_data)
    {
        if (numBytes >= recv_data_len)
        {
            numBytes = recv_data_len;
            memcpy(buffer, recv_data, numBytes);
            srcAddr = src_addr;
            recv_data = NULL;
            recv_data_len = 0;
            return true;
        }
        else
        {
            fprintf(stderr, "OpnetProtoSimProcess::UdpSocketProxy::RecvFrom buffer tool small\n");
            numBytes = 0;
            return false;    
        }        
    }
    else
    {
        fprintf(stderr, "OpnetProtoSimProcess::UdpSocketProxy::RecvFrom no data ready\n");
        numBytes = 0;
        return false;       
    }   
}  // end OpnetProtoSimProcess::UdpSocketProxy::RecvFrom()

bool OpnetProtoSimProcess::UdpSocketProxy::JoinGroup(const ProtoAddress& groupAddr)
{
    // (TBD)
	return true;
}  // end NsProtoSimAgent::SocketAgent::JoinGroup()

bool OpnetProtoSimProcess::UdpSocketProxy::LeaveGroup(const ProtoAddress& groupAddr)
{
    // (TBD)
	return true;
}  // end NsProtoSimAgent::SocketAgent::LeaveGroup()

void OpnetProtoSimProcess::UdpSocketProxy::OnReceive(char*               recvData, 
                                                     unsigned int        recvLen,
                                                     const ProtoAddress& srcAddr,
                                                     const ProtoAddress& dstAddr)
{
#ifdef OP_DEBUG1
printf("\tOpnetProtoSimProcess::UdpSocketProxy::OnReceive\n");
#endif
    // (TBD) filter out loopback multicast traffic
    recv_data = recvData;
    recv_data_len = recvLen;
    src_addr = srcAddr;
    if (proto_socket)
        proto_socket->OnNotify(ProtoSocket::NOTIFY_INPUT);
}  // end OpnetProtoSimProcess::UdpSocketProxy::OnReceive()


