#include "wxProtoApp.h"

/*BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_EVENT_TYPE(wxPROTO_DISPATCH_EVENT, -1)
END_DECLARE_EVENT_TYPES()

DEFINE_EVENT_TYPE(wxPROTO_DISPATCH_EVENT)*/

const wxEventType wxProtoApp::wxPROTO_DISPATCH_EVENT = wxNewEventType();

// it may also be convenient to define an event table macro for this event type
#define EVT_PROTO_DISPATCH_COMMAND(id, fn) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wxProtoApp::wxPROTO_DISPATCH_EVENT, id, -1, \
        (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&fn, \
        (wxObject *) NULL \
    ),

DECLARE_APP(wxProtoApp)

BEGIN_EVENT_TABLE(wxProtoApp, wxApp)
    EVT_PROTO_DISPATCH_COMMAND(-1, wxProtoApp::OnDispatchEvent)
END_EVENT_TABLE()


wxProtoApp::wxProtoApp()
 : ProtoDispatcher::Controller(dispatcher)
{
    
}

wxProtoApp::~wxProtoApp()
{
    dispatcher.Stop();
}

#define WXPROTO_USE_THREADING 1

int wxProtoApp::MainLoop()
{
#ifdef WXPROTO_USE_THREADING
    TRACE("starting dispatcher thread ...\n");
    // 1) Start ProtoDispatcher thread
    dispatcher.StartThread(this);

    TRACE("entering wxMainLoop() ...\n");
    
    // 2) Enter standard wxWidgets event loop
    int result = wxApp::MainLoop();
    
    TRACE("MainLoop exiting ...\n");
    
    // Stop ProtoDispatcher thread
    dispatcher.Stop();

    TRACE("dispatcher stopped ...\n");
    return result;
#else
    
#ifndef WIN32
    // The "guiTimer" makes sure we check for
    // wxWindows GUI events regularly
    // (TBD) asynchronously monitor for GUI events instead
    // (this already works on Win32)
    ProtoTimer guiTimer;  
    guiTimer.SetRepeat(-1);
    guiTimer.SetInterval(1.0);
    ActivateTimer(guiTimer);
#endif // !WIN32
   
    while (1)
    {
        TRACE("checking pending: ...\n");
        if (Pending())
        {
            TRACE("calling wxApp::Dispatch() ...\n");
            Dispatch();   
        }
        else
        {
#ifndef WIN32  // (Win32 ProtoDispatcher breaks on messages)
            if (dispatcher.IsPending())
#endif // WIN32
            {
                TRACE("waiting on the dispatcher ...\n");
                dispatcher.Wait();
                dispatcher.Dispatch();
            }
        }
        if (NULL == GetTopWindow())
        { 
#ifndef WIN32
            if (guiTimer.IsActive()) guiTimer.Deactivate();
#endif // !WIN32
            if (!dispatcher.IsPending())
            {   
                dispatcher.Stop();
                break;
            }            
        }
#ifndef WIN32
        else if (!guiTimer.IsActive())
        {
             ActivateTimer(guiTimer);
        }
#endif // !WIN32
    }
    return 0;
#endif
}  // end wxProtoApp::MainLoop()

bool wxProtoApp::SignalDispatchReady()
{
    wxCommandEvent dispatchEvent(wxPROTO_DISPATCH_EVENT);
    //AddPendingEvent(dispatchEvent);   
    wxPostEvent(this, dispatchEvent);
    TRACE("event posted.\n");
    return true;
}  // end wxProtoApp::SignalDispatchReady()


void wxProtoApp::OnDispatchEvent(wxCommandEvent& /*event*/)
{
    OnDispatch();
}
